@@ -33,7 +33,7 @@ if typing.TYPE_CHECKING:
     from libqtile.widget.base import _Widget
 
 
-class Gap(CommandObject):
+class Gap:
     """A gap placed along one of the edges of the screen
 
     If a gap has been defined, Qtile will avoid covering it with windows. The
@@ -108,30 +108,12 @@ class Gap(CommandObject):
     def geometry(self):
         return (self.x, self.y, self.width, self.height)
 
-    def _items(self, name: str) -> ItemT:
-        if name == "screen" and self.screen is not None:
-            return True, []
-        return None
-
-    def _select(self, name, sel):
-        if name == "screen":
-            return self.screen
-
     @property
     def position(self):
         for i in ["top", "bottom", "left", "right"]:
             if getattr(self.screen, i) is self:
                 return i
 
-    def info(self):
-        return dict(position=self.position)
-
-    def cmd_info(self):
-        """
-        Info for this object.
-        """
-        return self.info()
-
 
 class Obj:
     def __init__(self, name):
@@ -149,7 +131,7 @@ CALCULATED = Obj("CALCULATED")
 STATIC = Obj("STATIC")
 
 
-class Bar(Gap, configurable.Configurable):
+class Bar(Gap, configurable.Configurable, CommandObject):
     """A bar, which can contain widgets
 
     Parameters
@@ -179,6 +161,7 @@ class Bar(Gap, configurable.Configurable):
         self.window = None
         self.size_calculated = 0
         self._configured = False
+        self.has_keyboard: _Widget | None = None
 
         if isinstance(self.margin, int):
             self.margin = [self.margin] * 4
@@ -327,6 +310,16 @@ class Bar(Gap, configurable.Configurable):
 
     def _configure_widget(self, widget):
         configured = True
+
+        if widget.supported_backends and (self.qtile.core.name not in widget.supported_backends):
+            logger.warning(
+                "Widget removed: %s does not support %s.",
+                widget.__class__.__name__,
+                self.qtile.core,
+            )
+            self.crashed_widgets.append(widget)
+            return False
+
         try:
             widget._configure(self.qtile, self)
 
@@ -336,10 +329,9 @@ class Bar(Gap, configurable.Configurable):
                 widget.offsetx = self.border_width[3]
 
             widget.configured = True
-        except Exception as e:
-            logger.error(
-                "{} widget crashed during _configure with "
-                "error: {}".format(widget.__class__.__name__, repr(e))
+        except Exception:
+            logger.exception(
+                "%s widget crashed during _configure with error:", widget.__class__.__name__
             )
             self.crashed_widgets.append(widget)
             configured = False
@@ -352,13 +344,19 @@ class Bar(Gap, configurable.Configurable):
 
         for i in self.crashed_widgets:
             index = self.widgets.index(i)
-            crash = ConfigErrorWidget(widget=i)
-            crash._configure(self.qtile, self)
-            if self.horizontal:
-                crash.offsety = self.border_width[0]
-            else:
-                crash.offsetx = self.border_width[3]
-            self.widgets.insert(index, crash)
+
+            # Widgets that aren't available on the current backend should not
+            # be shown as "crashed" as the behaviour is expected. Only notify
+            # for genuine crashes.
+            if not i.supported_backends or (self.qtile.core.name in i.supported_backends):
+                crash = ConfigErrorWidget(widget=i)
+                crash._configure(self.qtile, self)
+                if self.horizontal:
+                    crash.offsety = self.border_width[0]
+                else:
+                    crash.offsetx = self.border_width[3]
+                self.widgets.insert(index, crash)
+
             self.widgets.remove(i)
 
     def _items(self, name: str) -> ItemT:
@@ -385,6 +383,10 @@ class Bar(Gap, configurable.Configurable):
 
     def kill_window(self):
         """Kill the window when the bar's screen is no longer being used."""
+        for name, w in self.qtile.widgets_map.copy().items():
+            if w in self.widgets:
+                w.finalize()
+                del self.qtile.widgets_map[name]
         self.drawer.finalize()
         self.window.kill()
         self.window = None
@@ -665,5 +667,11 @@ class Bar(Gap, configurable.Configurable):
         """
         self.process_button_click(x, y, button)
 
+    def cmd_info(self):
+        """
+        Info for this object.
+        """
+        return self.info()
+
 
 BarType = typing.Union[Bar, Gap]
